@@ -1,5 +1,7 @@
 #ifndef __KERNEL__
 
+#pragma GCC system_header
+
 #include_next <stdio.h>
 
 #ifndef _XENO_POSIX_STDIO_H
@@ -7,6 +9,7 @@
 
 #include <stddef.h>
 #include <stdarg.h>
+#include <xeno_config.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -14,6 +17,10 @@ extern "C" {
 
 int __real_vfprintf(FILE *stream, const char *fmt, va_list args);
 
+#ifdef CONFIG_XENO_FORTIFY
+int __real___vfprintf_chk(FILE *stream, int level, const char *fmt, va_list ap);
+#endif
+
 int __real_vprintf(const char *fmt, va_list args);
 
 int __real_fprintf(FILE *stream, const char *fmt, ...);
