@@ -322,7 +322,7 @@ static int rtswitch_set_tasks_count(rtswitch_context_t *ctx, unsigned count)
 	if (ctx->tasks_count == count)
 		return 0;
 
-	tasks = kmalloc(count * sizeof(*tasks), GFP_KERNEL);
+	tasks = vmalloc(count * sizeof(*tasks));
 
 	if (!tasks)
 		return -ENOMEM;
@@ -330,7 +330,7 @@ static int rtswitch_set_tasks_count(rtswitch_context_t *ctx, unsigned count)
 	down(&ctx->lock);
 
 	if (ctx->tasks)
-		kfree(ctx->tasks);
+		vfree(ctx->tasks);
 
 	ctx->tasks = tasks;
 	ctx->tasks_count = count;
@@ -543,7 +543,7 @@ static int rtswitch_close(struct rtdm_dev_context *context,
 				xnpod_delete_thread(&task->ktask);
 			rtdm_event_destroy(&task->rt_synch);
 		}
-		kfree(ctx->tasks);
+		vfree(ctx->tasks);
 	}
 	rtdm_timer_destroy(&ctx->wake_up_delay);
 	rtdm_nrtsig_destroy(&ctx->wake_utask);
@@ -749,6 +749,8 @@ int __init __switchtest_init(void)
 {
 	int err;
 
+	fp_features_init();
+
 	do {
 		snprintf(device.device_name, RTDM_MAX_DEVNAME_LEN,
 			 "rttest-switchtest%d",
