@@ -146,8 +146,7 @@ static void vfile_snapshot_free(struct xnvfile_snapshot_iterator *it, void *buf)
 
 static int vfile_snapshot_open(struct inode *inode, struct file *file)
 {
-	struct proc_dir_entry *pde = PDE(inode);
-	struct xnvfile_snapshot *vfile = pde->data;
+	struct xnvfile_snapshot *vfile = PDE_DATA(inode);
 	struct xnvfile_snapshot_ops *ops = vfile->ops;
 	struct xnvfile_snapshot_iterator *it;
 	int revtag, ret, nrdata;
@@ -324,8 +323,7 @@ static int vfile_snapshot_release(struct inode *inode, struct file *file)
 ssize_t vfile_snapshot_write(struct file *file, const char __user *buf,
 			     size_t size, loff_t *ppos)
 {
-	struct proc_dir_entry *pde = PDE(wrap_f_inode(file));
-	struct xnvfile_snapshot *vfile = pde->data;
+	struct xnvfile_snapshot *vfile = PDE_DATA(wrap_f_inode(file));
 	struct xnvfile_input input;
 	ssize_t ret;
 
@@ -420,16 +418,13 @@ int xnvfile_init_snapshot(const char *name,
 
 	mode = vfile->ops->store ? 0644 : 0444;
 	ppde = parent->entry.pde;
-	pde = create_proc_entry(name, mode, ppde);
+	pde = proc_create_data(name, mode, ppde, &vfile_snapshot_fops, vfile);
 	if (pde == NULL)
 		return -ENOMEM;
 
-	pde->proc_fops = &vfile_snapshot_fops;
 	wrap_proc_dir_entry_owner(pde);
 
-	vfile->entry.parent = parent;
 	vfile->entry.pde = pde;
-	pde->data = vfile;
 
 	return 0;
 }
@@ -513,8 +508,7 @@ static struct seq_operations vfile_regular_ops = {
 
 static int vfile_regular_open(struct inode *inode, struct file *file)
 {
-	struct proc_dir_entry *pde = PDE(inode);
-	struct xnvfile_regular *vfile = pde->data;
+	struct xnvfile_regular *vfile = PDE_DATA(inode);
 	struct xnvfile_regular_ops *ops = vfile->ops;
 	struct xnvfile_regular_iterator *it;
 	struct seq_file *seq;
@@ -582,8 +576,7 @@ static int vfile_regular_release(struct inode *inode, struct file *file)
 ssize_t vfile_regular_write(struct file *file, const char __user *buf,
 			    size_t size, loff_t *ppos)
 {
-	struct proc_dir_entry *pde = PDE(wrap_f_inode(file));
-	struct xnvfile_regular *vfile = pde->data;
+	struct xnvfile_regular *vfile = PDE_DATA(wrap_f_inode(file));
 	struct xnvfile_input input;
 	ssize_t ret;
 
@@ -658,16 +651,13 @@ int xnvfile_init_regular(const char *name,
 
 	mode = vfile->ops->store ? 0644 : 0444;
 	ppde = parent->entry.pde;
-	pde = create_proc_entry(name, mode, ppde);
+	pde = proc_create_data(name, mode, ppde, &vfile_regular_fops, vfile);
 	if (pde == NULL)
 		return -ENOMEM;
 
-	pde->proc_fops = &vfile_regular_fops;
 	wrap_proc_dir_entry_owner(pde);
 
-	vfile->entry.parent = parent;
 	vfile->entry.pde = pde;
-	pde->data = vfile;
 
 	return 0;
 }
@@ -703,11 +693,10 @@ int xnvfile_init_dir(const char *name,
 		parent = &sysroot;
 
 	ppde = parent->entry.pde;
-	pde = create_proc_entry(name, S_IFDIR, ppde);
+	pde = proc_mkdir(name, ppde);
 	if (pde == NULL)
 		return -ENOMEM;
 
-	vdir->entry.parent = parent;
 	vdir->entry.pde = pde;
 	vdir->entry.lockops = NULL;
 	vdir->entry.private = NULL;
@@ -755,7 +744,6 @@ int xnvfile_init_link(const char *from,
 	if (pde == NULL)
 		return -ENOMEM;
 
-	vlink->entry.parent = parent;
 	vlink->entry.pde = pde;
 	vlink->entry.lockops = NULL;
 	vlink->entry.private = NULL;
@@ -774,10 +762,7 @@ EXPORT_SYMBOL_GPL(xnvfile_init_link);
  */
 void xnvfile_destroy(struct xnvfile *vfile)
 {
-	struct proc_dir_entry *ppde;
-
-	ppde = vfile->parent ? vfile->parent->entry.pde : nkvfroot.entry.pde;
-	remove_proc_entry(vfile->pde->name, ppde);
+	proc_remove(vfile->pde);
 }
 EXPORT_SYMBOL_GPL(xnvfile_destroy);
 
@@ -971,11 +956,10 @@ int __init xnvfile_init_root(void)
 	struct xnvfile_directory *vdir = &nkvfroot;
 	struct proc_dir_entry *pde;
 
-	pde = create_proc_entry("xenomai", S_IFDIR, NULL);
+	pde = proc_mkdir("xenomai", NULL);
 	if (pde == NULL)
 		return -ENOMEM;
 
-	vdir->entry.parent = NULL;
 	vdir->entry.pde = pde;
 	vdir->entry.lockops = NULL;
 	vdir->entry.private = NULL;
