@@ -2,7 +2,7 @@
  * @file
  * This file is part of the Xenomai project.
  *
- * @note Copyright (C) 2004 Philippe Gerum <rpm@xenomai.org> 
+ * @note Copyright (C) 2004 Philippe Gerum <rpm@xenomai.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -52,33 +52,43 @@
 #include <nucleus/registry.h>
 #include <native/pipe.h>
 
-#ifdef CONFIG_PROC_FS
+#ifdef CONFIG_XENO_OPT_VFILE
 
-static ssize_t __pipe_link_proc(char *buf, int count, void *data)
+static char *__pipe_link_target(void *obj)
 {
-	RT_PIPE *pipe = (RT_PIPE *)data;
-	return snprintf(buf, count, "/dev/rtp%d", pipe->minor);
+	RT_PIPE *pipe = obj;
+	char *buf;
+
+	/* XXX: older kernels don't have kasprintf(). */
+	buf = kmalloc(32, GFP_KERNEL);
+	if (buf == NULL)
+		return buf;
+
+	snprintf(buf, 32, "/dev/rtp%d", pipe->minor);
+
+	return buf;
 }
 
-extern xnptree_t __native_ptree;
+extern struct xnptree __native_ptree;
 
-static xnpnode_t __pipe_pnode = {
-
-	.dir = NULL,
-	.type = "pipes",
-	.entries = 0,
-	.link_proc = &__pipe_link_proc,
-	.root = &__native_ptree,
+static struct xnpnode_link __pipe_pnode = {
+	.node = {
+		.dirname = "pipes",
+		.root = &__native_ptree,
+		.ops = &xnregistry_vlink_ops,
+	},
+	.target = __pipe_link_target,
 };
 
-#else /* !CONFIG_PROC_FS */
+#else /* !CONFIG_XENO_OPT_VFILE */
 
-static xnpnode_t __pipe_pnode = {
-
-	.type = "pipes"
+static struct xnpnode_link __pipe_pnode = {
+	.node = {
+		.dirname = "pipes",
+	},
 };
 
-#endif /* !CONFIG_PROC_FS */
+#endif /* !CONFIG_XENO_OPT_VFILE */
 
 static void __pipe_flush_pool(xnheap_t *heap,
 			      void *poolmem, u_long poolsize, void *cookie)
@@ -107,7 +117,7 @@ static void __pipe_free_handler(void *buf, void *xstate) /* nklock free */
 {
 	RT_PIPE *pipe = xstate;
 	spl_t s;
-	
+
 	if (buf == pipe->buffer) {
 		/* Reset the streaming buffer. */
 		xnlock_get_irqsave(&nklock, s);
@@ -131,7 +141,7 @@ static void __pipe_output_handler(xnpipe_mh_t *mh, void *xstate) /* nklock held
 static int __pipe_input_handler(xnpipe_mh_t *mh, int retval, void *xstate) /* nklock held */
 {
 	RT_PIPE *pipe = xstate;
-	
+
 	if (pipe->monitor == NULL)
 		return retval;
 
@@ -150,6 +160,8 @@ static void __pipe_release_handler(void *xstate) /* nklock free */
 
 	if (pipe->bufpool == &pipe->privpool)
 		xnheap_destroy(&pipe->privpool, __pipe_flush_pool, NULL);
+	else if (pipe->buffer)
+		xnheap_free(pipe->bufpool, pipe->buffer);
 
 #ifdef CONFIG_XENO_OPT_PERVASIVE
 	if (pipe->cpid)
@@ -318,7 +330,7 @@ int rt_pipe_create(RT_PIPE *pipe, const char *name, int minor, size_t poolsize)
 	xnpipe_m_size(pipe->buffer) = streamsz - sizeof(RT_PIPE_MSG);
 #endif /* CONFIG_XENO_OPT_NATIVE_PIPE_BUFSZ > 0 */
 
-			       
+
 	ops.output = &__pipe_output_handler;
 	ops.input = &__pipe_input_handler;
 	ops.alloc_ibuf = &__pipe_alloc_handler;
@@ -352,7 +364,7 @@ int rt_pipe_create(RT_PIPE *pipe, const char *name, int minor, size_t poolsize)
 	 */
 	if (name) {
 		err = xnregistry_enter(pipe->name, pipe, &pipe->handle,
-				       &__pipe_pnode);
+				       &__pipe_pnode.node);
 		if (err)
 			rt_pipe_delete(pipe);
 	}
@@ -699,9 +711,6 @@ ssize_t rt_pipe_read(RT_PIPE *pipe, void *buf, size_t size, RTIME timeout)
  *
  * - -EINVAL is returned if @a pipe is not a pipe descriptor.
  *
- * - -EPIPE is returned if the associated special device is not yet
- * open.
- *
  * - -EIDRM is returned if @a pipe is a closed pipe descriptor.
  *
  * - -ENODEV or -EBADF are returned if @a pipe is scrambled.
@@ -715,6 +724,11 @@ ssize_t rt_pipe_read(RT_PIPE *pipe, void *buf, size_t size, RTIME timeout)
  * - Kernel-based task
  *
  * Rescheduling: possible.
+ *
+ * @note Writing data to a pipe before any peer has opened the
+ * associated special device is allowed. The output will be buffered
+ * until then, only restricted by the available memory in the relevant
+ * buffer pool (see rt_pipe_create()).
  */
 
 ssize_t rt_pipe_send(RT_PIPE *pipe, RT_PIPE_MSG *msg, size_t size, int mode)
@@ -780,9 +794,6 @@ ssize_t rt_pipe_send(RT_PIPE *pipe, RT_PIPE_MSG *msg, size_t size, int mode)
  *
  * - -EINVAL is returned if @a pipe is not a pipe descriptor.
  *
- * - -EPIPE is returned if the associated special device is not yet
- * open.
- *
  * - -ENOMEM is returned if not enough buffer space is available to
  * complete the operation.
  *
@@ -800,6 +811,11 @@ ssize_t rt_pipe_send(RT_PIPE *pipe, RT_PIPE_MSG *msg, size_t size, int mode)
  * - User-space task
  *
  * Rescheduling: possible.
+ *
+ * @note Writing data to a pipe before any peer has opened the
+ * associated special device is allowed. The output will be buffered
+ * until then, only restricted by the available memory in the relevant
+ * buffer pool (see rt_pipe_create()).
  */
 
 ssize_t rt_pipe_write(RT_PIPE *pipe, const void *buf, size_t size, int mode)
@@ -856,9 +872,6 @@ ssize_t rt_pipe_write(RT_PIPE *pipe, const void *buf, size_t size, int mode)
  *
  * - -EINVAL is returned if @a pipe is not a pipe descriptor.
  *
- * - -EPIPE is returned if the associated special device is not yet
- * open.
- *
  * - -EIDRM is returned if @a pipe is a closed pipe descriptor.
  *
  * - -ENODEV or -EBADF are returned if @a pipe is scrambled.
@@ -877,6 +890,11 @@ ssize_t rt_pipe_write(RT_PIPE *pipe, const void *buf, size_t size, int mode)
  * - User-space task
  *
  * Rescheduling: possible.
+ *
+ * @note Writing data to a pipe before any peer has opened the
+ * associated special device is allowed. The output will be buffered
+ * until then, only restricted by the available memory in the relevant
+ * buffer pool (see rt_pipe_create()).
  */
 
 ssize_t rt_pipe_stream(RT_PIPE *pipe, const void *buf, size_t size)
@@ -1164,19 +1182,19 @@ int rt_pipe_monitor(RT_PIPE *pipe, int (*fn)(RT_PIPE *pipe, int event, long arg)
 	xnlock_put_irqrestore(&nklock, s);
 
 	return 0;
-	
+
 }
 
 /*@}*/
 
-EXPORT_SYMBOL(rt_pipe_create);
-EXPORT_SYMBOL(rt_pipe_delete);
-EXPORT_SYMBOL(rt_pipe_receive);
-EXPORT_SYMBOL(rt_pipe_send);
-EXPORT_SYMBOL(rt_pipe_read);
-EXPORT_SYMBOL(rt_pipe_write);
-EXPORT_SYMBOL(rt_pipe_stream);
-EXPORT_SYMBOL(rt_pipe_alloc);
-EXPORT_SYMBOL(rt_pipe_free);
-EXPORT_SYMBOL(rt_pipe_flush);
-EXPORT_SYMBOL(rt_pipe_monitor);
+EXPORT_SYMBOL_GPL(rt_pipe_create);
+EXPORT_SYMBOL_GPL(rt_pipe_delete);
+EXPORT_SYMBOL_GPL(rt_pipe_receive);
+EXPORT_SYMBOL_GPL(rt_pipe_send);
+EXPORT_SYMBOL_GPL(rt_pipe_read);
+EXPORT_SYMBOL_GPL(rt_pipe_write);
+EXPORT_SYMBOL_GPL(rt_pipe_stream);
+EXPORT_SYMBOL_GPL(rt_pipe_alloc);
+EXPORT_SYMBOL_GPL(rt_pipe_free);
+EXPORT_SYMBOL_GPL(rt_pipe_flush);
+EXPORT_SYMBOL_GPL(rt_pipe_monitor);
