@@ -422,7 +422,6 @@ static int __pthread_set_name_np(struct pt_regs *regs)
 {
 	char name[XNOBJECT_NAME_LEN];
 	struct pse51_hkey hkey;
-	struct task_struct *p;
 	pthread_t k_tid;
 
 	if (__xn_safe_strncpy_from_user(name,
@@ -435,11 +434,6 @@ static int __pthread_set_name_np(struct pt_regs *regs)
 	hkey.u_tid = __xn_reg_arg1(regs);
 	hkey.mm = current->mm;
 	k_tid = __pthread_find(&hkey);
-	if (k_tid) {
-		p = xnthread_user_task(&k_tid->threadbase);
-		strncpy(p->comm, name, sizeof(p->comm));
-		p->comm[sizeof(p->comm) - 1] = '\0';
-	}
 
 	return -pthread_set_name_np(k_tid, name);
 }
@@ -653,7 +647,7 @@ static int __sem_open(struct pt_regs *regs)
 	if (assoc) {
 		usm = assoc2usem(assoc);
 		++usm->refcnt;
-		xnlock_put_irqrestore(&nklock, s);
+		xnlock_put_irqrestore(&pse51_assoc_lock, s);
 		goto got_usm;
 	}
 
@@ -673,7 +667,7 @@ static int __sem_open(struct pt_regs *regs)
 	assoc = pse51_assoc_lookup(&q->usems, (u_long)sm->shadow_sem.sem);
 	if (assoc) {
 		assoc2usem(assoc)->refcnt++;
-		xnlock_put_irqrestore(&nklock, s);
+		xnlock_put_irqrestore(&pse51_assoc_lock, s);
 		xnfree(usm);
 		usm = assoc2usem(assoc);
 		goto got_usm;
