@@ -4,16 +4,16 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <sys/mman.h>
 
 #include <nucleus/heap.h>
 #include <asm/xenomai/syscall.h>
-#include <asm-generic/xenomai/bits/current.h>
+#include <asm-generic/xenomai/current.h>
 #include <asm-generic/xenomai/timeconv.h>
 #include <asm-generic/xenomai/stack.h>
 #include <asm/xenomai/bits/bind.h>
 #include "sem_heap.h"
 
-int xeno_sigxcpu_no_mlock = 1;
 static pthread_t xeno_main_tid;
 
 static void xeno_sigill_handler(int sig)
@@ -92,6 +92,11 @@ xeno_bind_skin_opt(unsigned skin_magic, const char *skin, const char *module)
 		exit(EXIT_FAILURE);
 	}
 
+	if (mlockall(MCL_CURRENT | MCL_FUTURE)) {
+		perror("Xenomai: mlockall");
+		exit(EXIT_FAILURE);
+	}
+
 	xeno_featinfo = finfo;
 	xeno_init_arch_features();
 
@@ -114,25 +119,3 @@ void xeno_fault_stack(void)
 		stk[0] = stk[sizeof(stk) - 1] = 0xA5;
 	}
 }
-
-void xeno_handle_mlock_alert(int sig, siginfo_t *si, void *context)
-{
-	struct sigaction sa;
-
-	if (si->si_value.sival_int == SIGDEBUG_NOMLOCK) {
-		fprintf(stderr, "Xenomai: process memory not locked "
-			"(missing mlockall?)\n");
-		fflush(stderr);
-		exit(4);
-	}
-
-	/* XNTRAPSW was set for the thread but no user-defined handler
-	   has been set to override our internal handler, so let's
-	   invoke the default signal action. */
-
-	sa.sa_handler = SIG_DFL;
-	sigemptyset(&sa.sa_mask);
-	sa.sa_flags = 0;
-	sigaction(SIGXCPU, &sa, NULL);
-	pthread_kill(pthread_self(), SIGXCPU);
-}
